unit Help;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, ExtCtrls, DsgnIntf;

type
  THelpForm = class(TForm)
    Frame: TPanel;
    Comment: TLabel;
    Subject: TLabel;
    procedure FormDeactivate(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure CommentClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    { Private-dclarations }
  public
    { Public-dclarations }
  end;

  THelpMgr = class(TComponent)
  private
   MsgHook: HHook;		{ Handle identifying our hook function }
   MsgInstance: TFarProc;
   fCount:integer;
   Procedure SetCount(Value:integer);
  protected
   Constructor Create(AOwner:TComponent); override;
   Destructor Destroy; override;
   Property Count:integer read fCount write SetCount;
  end;

  THelp = class(TComponent)
   private
    fSubject:string;
    fComment:string;
   protected
    Constructor Create(AOwner:TComponent); override;
    Destructor Destroy; override;
   public
    Procedure Show;
   published
    Property Subject:string read fSubject write fSubject;
    Property Comment:string read fComment write fComment;
  end;

var
  HelpForm: THelpForm;

Procedure Register;

implementation

{$R *.DFM}

{ THelpForm }

Var
 HelpMgr: THelpMgr;

function MsgFilter(Code : integer; wParam: word; lParam: longInt) : longInt; export;
 Var
  i:integer;
 begin
  If (Code<0)
  or (wParam<>VK_F1)
  then
   Result := CallNextHookEx(HelpMgr.MsgHook, Code, wParam, lParam)
  else begin
   Result:=1;
   if Screen.ActiveForm<>nil then
    With Screen.ActiveForm do
     For i:=0 to ComponentCount-1 do
      If Components[i] is THelp then begin THelp(Components[i]).Show; break; end;
  end;
end;

{ THelpMgr }
Constructor THelpMgr.Create(AOwner:TComponent);
 begin
  inherited Create(AOwner);
  fCount:=0;
  HelpForm:=nil;
 end;

Destructor THelpMgr.Destroy;
 begin
  Count:=0;
  Inherited Destroy;
 end;

Procedure THelpMgr.SetCount(Value:Integer);
 begin
  if fCount=Value then exit;
  fCount:=Value;
  if Value=0 then begin
   UnhookWindowsHookEx(MsgHook);
   FreeProcInstance(MsgInstance);
   {HelpForm.Free;}
  end else begin
   HelpForm:=THelpForm.Create(Application);
   MsgInstance := MakeProcInstance(@MsgFilter,hInstance);
   MsgHook := SetWindowsHookEx(WH_KEYBOARD, THookProc (MsgInstance), hInstance, GetCurrentTask);
  end;
 end;

{ THelp }

Constructor THelp.Create(AOwner:TComponent);
 begin
  inherited Create(AOwner);
  fSubject:='';
  fComment:='';
  HelpMgr.Count:=HelpMgr.Count+1;
 end;

Destructor THelp.Destroy;
 begin
  HelpMgr.Count:=HelpMgr.Count-1;
  inherited Destroy;
 end;

Procedure THelp.Show;
 Var
  s:string;
  p:byte;
 begin
  if Subject='' then Subject:=TForm(Owner).Caption;
  HelpForm.Subject.Caption:=Subject;
  s:=Comment;
  p:=pos('/n',s); while p>0 do begin s[p]:=#13; s[p+1]:=#10; p:=pos('/n',s); end;
  HelpForm.Comment.Caption:=s;
  HelpForm.Show;
 end;

{ THelpForm }

procedure THelpForm.FormDeactivate(Sender: TObject);
 begin
  Hide;
 end;

procedure THelpForm.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
 Hide;
end;

procedure THelpForm.CommentClick(Sender: TObject);
begin
 Hide;
end;

procedure THelpForm.FormShow(Sender: TObject);
begin
 Width:=Comment.Width+2*Frame.BorderWidth;
 Height:=Subject.Height+Comment.Height+2*Frame.BorderWidth;
 If Application.MainForm<>nil then begin
  Left:=Application.MainForm.Left+(Application.MainForm.Width-Width) div 2;
  Top:=Application.MainForm.Top+(Application.MainForm.Height-Height) div 2;
 end else begin
  Left:=(Screen.Width-Width) div 2;
  Top:=(Screen.Height-Height) div 2;
 end;
end;

{ Registration }
Type
 THelpEditor= class(TComponentEditor)
  Procedure Edit; override;
 end;

Procedure THelpEditor.Edit;
 begin
  THelp(Component).Show;
 end;

Procedure Register;
 begin
  RegisterComponents('Exemples',[THelp]);
  RegisterComponentEditor(THelp,THelpEditor);
 end;

begin
 HelpMgr:=THelpMgr.Create(Application);
end.
